def get_peak(img, map, n_smooth=50, axis=0):
    '''
    '''
    from scipy.signal import find_peaks
    import numpy as np
    # compute signal
    ls_mean = img.mean(axis=(not axis)*1) # 0:nrow
    # gaussian smooth signal
    for i in range(n_smooth):
        ls_mean = np.convolve(np.array([1, 2, 4, 2, 1])/10, ls_mean, mode='same')
    peaks, _ = find_peaks(ls_mean)
    if map is not None:
        if len(peaks) > map.shape[axis]:
            while len(peaks) > map.shape[axis]:
                ls_diff = [peaks[i+1]-peaks[i] for i in range(len(peaks)-1)]
                idx_diff = np.argmin(ls_diff)
                idx_kick = idx_diff if (ls_mean[peaks[idx_diff]] < ls_mean[peaks[idx_diff+1]]) else (idx_diff+1)
                peaks = np.delete(peaks, idx_kick)
        elif len(peaks) < map.shape[axis]:
            while len(peaks) < map.shape[axis]:
                ls_diff = [peaks[i+1]-peaks[i] for i in range(len(peaks)-1)]
                idx_diff = np.argmax(ls_diff)
                peak_insert = (peaks[idx_diff]+peaks[idx_diff+1])/2
                peaks = np.sort(np.append(peaks, int(peak_insert)))
    return peaks, ls_mean

def read_jpg(filename):
    import numpy as np
    from PIL import Image
    Image.MAX_IMAGE_PIXELS = 1e+9
    img = np.array(Image.open(filename)).astype(np.uint8)
    return img

def read_tiff(filename, bands=None, xBSize=5000, yBSize=5000):
    '''import'''
    import gdal
    import numpy as np
    from tqdm import tqdm_gui
    '''program'''
    ds = gdal.Open(filename)
    gdal.UseExceptions()
    nrow = ds.RasterYSize
    ncol = ds.RasterXSize
    if bands==None:
        bands = range(ds.RasterCount)
    data = np.zeros((nrow, ncol, len(bands)))
    for b in bands:
        band = ds.GetRasterBand(b+1)
        for i in tqdm_gui(range(0, nrow, yBSize), desc="Channel %d/%d"%(b, len(bands)-1), leave=False):
            if i + yBSize < nrow:
                numRows = yBSize
            else:
                numRows = nrow - i
            for j in range(0, ncol, xBSize):
                if j + xBSize < ncol:
                    numCols = xBSize
                else:
                    numCols = ncol - j
                data[i:(i+numRows), j:(j+numCols), b] = band.ReadAsArray(j, i, numCols, numRows)
    return data.astype(np.uint8)

def write_tiff(array, outname):
    driver = gdal.GetDriverByName("GTiff")
    out_info = driver.Create(outname+".tif",
                   array.shape[1], # x
                   array.shape[0], # y
                   array.shape[2], # channels
                   gdal.GDT_Byte)
    for i in range(array.shape[2]):
        out_info.GetRasterBand(i+1).WriteArray(array[:,:,i])



import numpy as np
from PIL import Image
Image.MAX_IMAGE_PIXELS = 1e+9
from PyQt5.QtWidgets import *
from PyQt5.QtCore import *
from PyQt5.QtGui import *

class Panel_Input(QWidget):
    def __init__(self):
        super().__init__()
        # QGroupBox{border-radius:px}
        '''attr'''
        # user define
        self.gr_user = QGroupBox("User's Input")
        self.lo_user = QGridLayout()
        self.lb_img = QLabel()
        self.lb_map = QLabel()
        self.fd_img = QLineEdit()
        self.fd_map = QLineEdit()
        self.bt_img = QPushButton()
        self.bt_map = QPushButton()
        # demo
        self.gr_demo = QGroupBox("Demo")
        self.lo_demo = QVBoxLayout()
        self.lb_demo = QLabel("Will use sample files to demo the program")
        # self
        self.layout = QVBoxLayout()
        '''initialize UI'''
        self.initUI()
    def initUI(self):
        '''user'''
        # GUI components
        self.gr_user.setCheckable(True)
        self.gr_user.setChecked(False)
        self.gr_user.clicked.connect(lambda: self.toggle(self.gr_user))
        self.lb_img.setText("Image (.tif, .jpg, .png):")
        self.lb_map.setText("Map (.csv, .txt)(OPTIONAL):")
        font = self.fd_img.font()
        font.setPointSize(25)
        fm = QFontMetrics(font)
        self.fd_img.setFixedHeight(fm.height())
        self.fd_map.setFixedHeight(fm.height())
        self.bt_img.setText("Browse")
        self.bt_img.clicked.connect(self.assign_PathImg)
        self.bt_map.setText("Browse")
        self.bt_map.clicked.connect(self.assign_PathMap)
        # layout
        self.lo_user.addWidget(self.lb_img, 0, 0)
        self.lo_user.addWidget(self.fd_img, 0, 1)
        self.lo_user.addWidget(self.bt_img, 0, 2)
        self.lo_user.addWidget(self.lb_map, 1, 0)
        self.lo_user.addWidget(self.fd_map, 1, 1)
        self.lo_user.addWidget(self.bt_map, 1, 2)
        self.gr_user.setLayout(self.lo_user)
        '''demo'''
        # GUI components
        self.gr_demo.setCheckable(True)
        self.gr_demo.setChecked(True)
        self.gr_demo.clicked.connect(lambda: self.toggle(self.gr_demo))
        # layout
        self.lo_demo.addWidget(self.lb_demo)
        self.gr_demo.setLayout(self.lo_demo)
        '''layout'''
        self.layout.setContentsMargins(400, 50, 400, 50)
        self.layout.addWidget(self.gr_user)
        self.layout.addWidget(self.gr_demo)
        '''finalize'''
        self.setLayout(self.layout)
        self.show()
    def toggle(self, groupbox):
        if (groupbox.title()=="Demo"):
            self.gr_user.setChecked(not self.gr_user.isChecked())
        elif (groupbox.title()!="Demo"):
            self.gr_demo.setChecked(not self.gr_demo.isChecked())
    def assign_PathImg(self):
        fileter = "Images (*.tif *.jpg *.jpeg *.png)"
        path = QFileDialog().getOpenFileName(self, "", "", fileter)[0]
        self.fd_img.setText(path)
    def assign_PathMap(self):
        fileter = "Map (*.csv *.txt)"
        path = QFileDialog().getOpenFileName(self, "", "", fileter)[0]
        self.fd_map.setText(path)
    def get_img(self):
        import pandas as pd
        import cv2
        import io
        import urllib.request
        import rasterio
        if self.gr_user.isChecked():
            # user input
            ras = rasterio.open(self.fd_img.text())
            nCh = ras.count
            if nCh<3:
                img_np = np.zeros((ras.height, ras.width, 3))
                for i in range(3):
                    img_np[:,:,i] = ras.read(1)
            else:
                img_np = np.zeros((ras.height, ras.width, nCh))
                for i in range(nCh):
                    img_np[:,:,i] = ras.read(i+1)
            img = img_np.astype(np.uint8)
            # map
            try:
                map = pd.read_csv(self.fd_map.text(), header=None)
            except:
                map = None
        else:
            # demo
            path_map = "http://www.zzlab.net/James_Demo/seg_map.csv"
            path_img = "http://www.zzlab.net/James_Demo/seg_img.jpg"
            map = pd.read_csv(path_map, header=None)
            with urllib.request.urlopen(path_img) as url:
                file = io.BytesIO(url.read())
                img = np.array(Image.open(file))
            # demo 2
            # img = read_jpg("/Users/jameschen/Dropbox/James Chen/GRID/Modeling/Rhombus.jpg")
            # map = pd.read_csv("/Users/jameschen/Dropbox/James Chen/GRID/Modeling/Rhombus_Solution.csv", header=None)
        return img, map

def read_jpg(filename):
    img = np.array(Image.open(filename)).astype(np.uint8)
    return img
#
def read_tiff(filename, bands=None, xBSize=5000, yBSize=5000):
    '''import'''
    import gdal
    import numpy as np
    from tqdm import tqdm_gui
    '''program'''
    ds = gdal.Open(filename)
    gdal.UseExceptions()
    nrow = ds.RasterYSize
    ncol = ds.RasterXSize
    if bands==None:
        bands = range(ds.RasterCount)
    data = np.zeros((nrow, ncol, len(bands)))
    for b in bands:
        band = ds.GetRasterBand(b+1)
        for i in tqdm_gui(range(0, nrow, yBSize), desc="Channel %d/%d"%(b, len(bands)-1), leave=False):
            if i + yBSize < nrow:
                numRows = yBSize
            else:
                numRows = nrow - i
            for j in range(0, ncol, xBSize):
                if j + xBSize < ncol:
                    numCols = xBSize
                else:
                    numCols = ncol - j
                data[i:(i+numRows), j:(j+numCols), b] = band.ReadAsArray(j, i, numCols, numRows)
    # data = (data - data.min(axis=(0,1)))/(data.max(axis=(0,1))-data.min(axis=(0,1)))
    return data.astype(np.uint8)



import os, sys
from PIL import Image
from enum import Enum
# os.chdir(os.path.expanduser("~")+"/Dropbox/James_Git/PyPi/AgricolAi/AgricolAi/Field_Segmentation")
from .GUI_Input import *
from .GUI_Cropper import *
from .GUI_Kmeaner import *
from .GUI_Anchor import *
from .GUI_Output import *
from PyQt5.QtWidgets import *
from PyQt5.QtCore import *
from PyQt5.QtGui import *

class Panels(Enum):
    INPUT=0
    CROPPER=1
    KMEANER=2
    ANCHOR=3
    OUTPUT=4

class Window_Main(QMainWindow):
    def __init__(self):
        super().__init__()
        self.setStyleSheet("""
        QWidget {\
            font: 20pt Trebuchet MS
        }
        QGroupBox::title{
            subcontrol-origin: margin;
            left: 10px;
            padding: 0 3px 0 3px;
        }
        QGroupBox {
            border: 1px solid gray;
            border-radius: 9px;
            margin-top: 0.5em;
        }
        """)
        '''attr'''
        # GUI components
        self.pn_content = QWidget()
        self.pn_main = QStackedWidget()
        self.pn_navi = QWidget()
        self.bt_next = QPushButton()
        self.bt_back = QPushButton()
        self.layout = None
        # params
        self.params = dict()
        # image-related
        self.img_raw = None
        self.img_crop = None
        self.img_bin = None
        self.k_center = None
        # info
        self.title = "GRID"
        self.width = 1440
        self.height = 720
        '''initialize UI'''
        self.initUI()
    def initUI(self):
        self.setWindowTitle(self.title)
        self.setMinimumSize(self.width, self.height)
        '''first input'''
        self.show_input()
        '''set up windows'''
        center = QApplication.desktop().availableGeometry().center()
        rect = self.geometry()
        rect.moveCenter(center)
        self.setGeometry(rect)
    def show_input(self, isNewImg=True):
        '''input panel'''
        if isNewImg:
            self.pn_main.addWidget(Panel_Input())
        else:
            self.pn_main.removeWidget(self.pn_main.widget(Panels.CROPPER.value))
        self.pn_main.setCurrentIndex(Panels.INPUT.value)
        '''navigation bar'''
        self.assemble_navigation(name_next="Load Files ->", oneSide=True)
        self.bt_next.clicked.connect(lambda: self.show_cropper(isNewImg=True))
        '''finalize'''
        self.assemble_and_show()
    def show_cropper(self, isNewImg=True):
        '''input panel'''
        if isNewImg:
            self.params['raw'], self.params['map'] = self.pn_main.widget(Panels.INPUT.value).get_img()
            self.pn_main.addWidget(Panel_Cropper(np_img=self.params['raw']))
        else:
            self.pn_main.removeWidget(self.pn_main.widget(Panels.KMEANER.value))
        self.pn_main.setCurrentIndex(Panels.CROPPER.value)
        '''navigation bar'''
        self.assemble_navigation()
        self.bt_back.clicked.connect(lambda: self.show_input(isNewImg=False))
        self.bt_next.clicked.connect(lambda: self.show_kmeaner(isNewImg=True))
        '''finalize'''
        self.assemble_and_show()
    def show_kmeaner(self, isNewImg=True):
        '''input panel'''
        if isNewImg:
            self.params['crop'] = self.pn_main.widget(Panels.CROPPER.value).get_img()
            self.pn_main.addWidget(Panel_Kmeaner(np_img=self.params['crop']))
        else:
            self.pn_main.removeWidget(self.pn_main.widget(Panels.ANCHOR.value))
        self.pn_main.setCurrentIndex(Panels.KMEANER.value)
        '''navigation bar'''
        self.assemble_navigation()
        self.bt_back.clicked.connect(lambda: self.show_cropper(isNewImg=False))
        self.bt_next.clicked.connect(lambda: self.show_anchor(isNewImg=True))
        '''finalize'''
        self.assemble_and_show()
    def show_anchor(self, isNewImg=True):
        '''input panel'''
        if isNewImg:
            self.params['crop'], self.params['k'], self.params['bin'], self.params['ch_nir'], self.params['ch_red'], self.params['ls_bin'] = self.pn_main.widget(Panels.KMEANER.value).get_img()
            self.pn_main.addWidget(Panel_Anchor(img=self.params['bin'], map=self.params['map']))
        else:
            self.pn_main.removeWidget(self.pn_main.widget(Panels.OUTPUT.value))
        self.pn_main.setCurrentIndex(Panels.ANCHOR.value)
        '''navigation bar'''
        self.assemble_navigation()
        self.bt_back.clicked.connect(lambda: self.show_kmeaner(isNewImg=False))
        self.bt_next.clicked.connect(lambda: self.show_output(isNewImg=True))
        '''finalize'''
        self.assemble_and_show()
    def show_output(self, isNewImg=True):
        '''input panel'''
        if isNewImg:
            self.params['anchors'], self.params['nc'], self.params['nr'] = self.pn_main.widget(Panels.ANCHOR.value).get_anchors()
            self.pn_main.addWidget(Panel_Output(**self.params))
        self.pn_main.setCurrentIndex(Panels.OUTPUT.value)
        # test
        self.test()
        # test
        '''navigation bar'''
        self.assemble_navigation(name_next="Finish")
        self.bt_back.clicked.connect(lambda: self.show_anchor(isNewImg=False))
        self.bt_next.clicked.connect(self.finish)
        '''finalize'''
        self.assemble_and_show()
    def finish(self):
        msgBox = QMessageBox()
        msgBox.setIcon(QMessageBox.Information)
        msgBox.setText("Start another job?")
        msgBox.setWindowTitle("Finish")
        msgBox.setStandardButtons(QMessageBox.Cancel | QMessageBox.Yes | QMessageBox.Save)
        returnValue = msgBox.exec()
        if returnValue == QMessageBox.Yes:
            self.pn_main.widget(Panels.OUTPUT.value).output()
            self.show_input()
        elif returnValue == QMessageBox.Save:
            self.pn_main.widget(Panels.OUTPUT.value).output()
            self.show_output(isNewImg=False)
    def assemble_navigation(self, name_next="Next ->", name_back="<- Back", oneSide=False):
        self.pn_navi = QWidget()
        self.bt_next = QPushButton(name_next)
        self.bt_back = QPushButton(name_back)
        layout_navi = QHBoxLayout()
        if oneSide:
            layout_navi.addStretch(1)
        else:
            layout_navi.addWidget(self.bt_back)
        layout_navi.addWidget(self.bt_next)
        self.pn_navi.setLayout(layout_navi)
    def assemble_and_show(self):
        self.layout = QVBoxLayout()
        self.layout.addWidget(self.pn_main, Qt.AlignCenter)
        self.layout.addWidget(self.pn_navi)
        self.pn_content = QWidget()
        self.pn_content.setLayout(self.layout)
        self.setCentralWidget(self.pn_content)
        self.show()
    def test(self):
        import json
        with open('anchors', 'w') as fout:
            json.dump(self.params['anchors'], fout)
        np.save("img_crop", self.params['crop'])
        np.save("img_bin", self.params['bin'])
        np.save("map", self.params['map'])
        print("nc:%d"%(self.params['nc']))
        print("nr:%d"%(self.params['nr']))



import numpy as np
from .GUI_ClassFun import *
from PyQt5.QtWidgets import *
from PyQt5.QtCore import *
from PyQt5.QtGui import *

class Panel_Kmeaner(QWidget):
    def __init__(self, np_img):
        super().__init__()
        self.setFocusPolicy(Qt.StrongFocus)
        self.setFocus()
        self.update()
        '''attr'''
        # main/img
        self.layout = QGridLayout()
        # img preview (left)
        self.gr_left = QGroupBox("Corrected Image")
        self.lo_left = QGridLayout()
        self.wg_img = Widget_Kmeans(np_img)
        self.bt_ccw = QPushButton("rotate ccw (Q)")
        self.bt_cw = QPushButton("rorate cw (E)")
        # K mean (right)
        self.gr_pre = QGroupBox("K-means Algo.")
        self.lo_pre = QVBoxLayout()
        self.gr_ch = QGroupBox("Channel")
        self.lo_ch = QGridLayout()
        self.lb_nir = QLabel("NIR")
        self.cb_nir = QComboBox()
        self.lb_red = QLabel("Red")
        self.cb_red = QComboBox()
        self.gr_k = QGroupBox("K = 3")
        self.lo_k = QVBoxLayout()
        self.sl_k = QSlider(Qt.Horizontal)
        # Binarization
        self.gr_bin = QGroupBox("Binarization")
        self.lo_bin = QVBoxLayout()
        self.gr_cut = QGroupBox("Auto cutoff = 1")
        self.lo_cut = QVBoxLayout()
        self.sl_cut = QSlider(Qt.Horizontal)
        self.gr_cusb = QGroupBox("Custom")
        self.lo_cusb = QHBoxLayout()
        self.ck_cusb = []
        for i in range(1, 7):
            checkbox = QCheckBox(str(i))
            checkbox.stateChanged.connect(self.append_cut)
            if i>3:
                checkbox.setEnabled(False)
            self.ck_cusb.extend([checkbox])
        self.ls_bin = [0]
        # Display
        self.gr_dis = QGroupBox("Display")
        self.lo_dis = QHBoxLayout()
        self.rb_bin = QRadioButton("Binary (A)")
        self.rb_rgb = QRadioButton("RGB (S)")
        self.rb_k = QRadioButton("K-Means (D)")
        # refine (right)
        self.gr_pro = QGroupBox("Clusters Refine")
        self.lo_pro = QVBoxLayout()
        self.gr_shad = QGroupBox("De-Shadow = 0")
        self.lo_shad = QVBoxLayout()
        self.sl_shad = QSlider(Qt.Horizontal)
        self.gr_gb = QGroupBox("De-Noise = 0")
        self.lo_gb = QVBoxLayout()
        self.sl_gb = QSlider(Qt.Horizontal)
        '''initialize UI'''
        self.initUI()
    def initUI(self):
        '''img preview (left)'''
        # components
        self.bt_ccw.clicked.connect(self.wg_img.rotate_CCW)
        self.bt_cw.clicked.connect(self.wg_img.rotate_CW)
        # layout
        self.lo_left.addWidget(self.wg_img, 0, 0, 1, 2)
        self.lo_left.addWidget(self.bt_ccw, 1, 0)
        self.lo_left.addWidget(self.bt_cw, 1, 1)
        self.gr_left.setLayout(self.lo_left)
        '''pre keans (right)'''
        # components
        self.sl_k.setMinimum(2)
        self.sl_k.setMaximum(6)
        self.sl_k.setValue(3)
        self.sl_k.setTickInterval(1)
        self.sl_k.setTickPosition(QSlider.TicksBelow)
        self.sl_k.valueChanged.connect(self.change_k)
        for i in range(self.wg_img.imgC):
            self.cb_nir.addItem(str(i))
            self.cb_red.addItem(str(i))
        if self.wg_img.imgC>3:
            self.cb_nir.setCurrentIndex(3)
        else:
            self.cb_nir.setCurrentIndex(1)
        self.cb_nir.currentIndexChanged.connect(self.change_k)
        self.cb_red.currentIndexChanged.connect(self.change_k)
        # layout
        self.lo_ch.addWidget(self.lb_nir, 0, 0)
        self.lo_ch.addWidget(self.lb_red, 0, 1)
        self.lo_ch.addWidget(self.cb_nir, 1, 0)
        self.lo_ch.addWidget(self.cb_red, 1, 1)
        self.gr_ch.setLayout(self.lo_ch)
        self.gr_ch.setSizePolicy(QSizePolicy(QSizePolicy.Preferred, QSizePolicy.Fixed))
        self.lo_k.addWidget(self.sl_k)
        self.gr_k.setLayout(self.lo_k)
        self.lo_pre.addWidget(self.gr_ch)
        self.lo_pre.addWidget(self.gr_k)
        self.gr_pre.setLayout(self.lo_pre)
        '''binarization'''
        # component
        self.gr_cut.setCheckable(True)
        self.gr_cut.setChecked(True)
        self.gr_cut.clicked.connect(lambda: self.toggle(self.gr_cut))
        self.sl_cut.setMinimum(1)
        self.sl_cut.setMaximum(3)
        self.sl_cut.setValue(1)
        self.sl_cut.setTickInterval(1)
        self.sl_cut.setTickPosition(QSlider.TicksBelow)
        self.sl_cut.valueChanged.connect(self.change_cut)
        self.gr_cusb.setCheckable(True)
        self.gr_cusb.setChecked(False)
        self.gr_cusb.clicked.connect(lambda: self.toggle(self.gr_cusb))
        # layout
        self.lo_cut.addWidget(self.sl_cut)
        self.gr_cut.setLayout(self.lo_cut)
        for i in range(6):
            self.lo_cusb.addWidget(self.ck_cusb[i])
        self.gr_cusb.setLayout(self.lo_cusb)
        self.lo_bin.addWidget(self.gr_cut)
        self.lo_bin.addWidget(self.gr_cusb)
        self.gr_bin.setLayout(self.lo_bin)
        '''pro keans (right)'''
        # components
        self.sl_shad.setMinimum(0)
        self.sl_shad.setMaximum(255)
        self.sl_shad.setValue(0)
        self.sl_shad.setTickInterval(20)
        self.sl_shad.setTickPosition(QSlider.TicksBelow)
        self.sl_shad.valueChanged.connect(self.change_shad)
        self.sl_gb.setMinimum(0)
        self.sl_gb.setMaximum(50)
        self.sl_gb.setValue(0)
        self.sl_gb.setTickInterval(5)
        self.sl_gb.setTickPosition(QSlider.TicksBelow)
        self.sl_gb.valueChanged.connect(self.change_gb)
        # layout
        self.lo_shad.addWidget(self.sl_shad)
        self.gr_shad.setLayout(self.lo_shad)
        self.lo_gb.addWidget(self.sl_gb)
        self.gr_gb.setLayout(self.lo_gb)
        self.lo_pro.addWidget(self.gr_shad)
        self.lo_pro.addWidget(self.gr_gb)
        self.gr_pro.setLayout(self.lo_pro)
        '''display'''
        # components
        self.rb_bin.setChecked(True)
        self.rb_bin.toggled.connect(self.wg_img.switch_imgB)
        self.rb_rgb.toggled.connect(self.wg_img.switch_imgVis)
        self.rb_k.toggled.connect(self.wg_img.switch_imgK)
        # layout
        self.lo_dis.addWidget(self.rb_bin)
        self.lo_dis.addWidget(self.rb_rgb)
        self.lo_dis.addWidget(self.rb_k)
        self.gr_dis.setLayout(self.lo_dis)
        '''assemble'''
        policy_right = QSizePolicy(QSizePolicy.Preferred, QSizePolicy.Preferred)
        policy_right.setHorizontalStretch(1)
        self.gr_pre.setSizePolicy(policy_right)
        self.gr_bin.setSizePolicy(policy_right)
        self.gr_dis.setSizePolicy(policy_right)
        self.gr_pro.setSizePolicy(policy_right)
        policy_left = QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
        policy_left.setHorizontalStretch(3)
        self.gr_left.setSizePolicy(policy_left)
        self.layout.addWidget(self.gr_left, 0, 0, 4, 1)
        self.layout.addWidget(self.gr_pre, 0, 1)
        self.layout.addWidget(self.gr_bin, 1, 1)
        self.layout.addWidget(self.gr_pro, 2, 1)
        self.layout.addWidget(self.gr_dis, 3, 1)
        self.setLayout(self.layout)
        self.do_kmeans()
        self.wg_img.set_binarize([0])
        self.show()
    def toggle(self, groupbox):
        if (groupbox.title()=="Custom"):
            self.gr_cut.setChecked(not self.gr_cut.isChecked())
        else:
            self.gr_cusb.setChecked(not self.gr_cusb.isChecked())
        if self.gr_cusb.isChecked():
            self.append_cut()
        else:
            self.change_cut()
    def change_k(self):
        value = self.sl_k.value()
        self.sl_cut.setMaximum(value)
        self.gr_k.setTitle("K = %d" % value)
        self.do_kmeans()
        if self.gr_cusb.isChecked():
            self.append_cut()
        else:
            self.change_cut()
        self.refresh()
    def change_cut(self):
        value = self.sl_cut.value()
        self.gr_cut.setTitle("Auto cutoff = %d" % value)
        self.ls_bin = []
        for i in range(6):
            self.ck_cusb[i].setEnabled(False)
            if i<value:
                self.ls_bin.extend([i])
        self.wg_img.set_binarize(list=self.ls_bin)
    def append_cut(self):
        value = self.sl_k.value()
        self.ls_bin = []
        for i in range(6):
            if i<value:
                self.ck_cusb[i].setEnabled(True)
                if self.ck_cusb[i].isChecked():
                    self.ls_bin.extend([i])
            else:
                self.ck_cusb[i].setEnabled(False)
        self.wg_img.set_binarize(list=self.ls_bin)
    def change_shad(self):
        value = self.sl_shad.value()
        self.gr_shad.setTitle("De-Shadow = %d" % value)
        # self.sl_shad.setValue(0)
        self.wg_img.set_deshadow(value=value)
    def change_gb(self):
        value = self.sl_gb.value()
        self.gr_gb.setTitle("De-Noise = %d" % value)
        self.wg_img.set_smooth(value=value)
    def do_kmeans(self):
        # panel launch kmeans
        self.wg_img.do_kmeans(k=self.sl_k.value(), ch_NIR=int(self.cb_nir.currentText()), ch_Red=int(self.cb_red.currentText()))
    def keyPressEvent(self, event):
        if event.key() == Qt.Key_A:
            self.rb_bin.setChecked(True)
        elif event.key() == Qt.Key_S:
            self.rb_rgb.setChecked(True)
        elif event.key() == Qt.Key_D:
            self.rb_k.setChecked(True)
        elif event.key() == Qt.Key_Q:
            self.wg_img.rotate_CCW()
        elif event.key() == Qt.Key_E:
            self.wg_img.rotate_CW()
    # def keyReleaseEvent(self, event):
        # self.rb_bin.setChecked(True)
    def refresh(self):
        if self.rb_bin.isChecked():
            self.wg_img.switch_imgB()
        elif self.rb_rgb.isChecked():
            self.wg_img.switch_imgVis()
        elif self.rb_k.isChecked():
            self.wg_img.switch_imgK()
    def get_img(self):
        ch_NIR = int(self.cb_nir.currentText())
        ch_Red = int(self.cb_red.currentText())
        img_raw, img_k, img_bin = self.wg_img.getImages()
        ls_bin = self.wg_img.ls_bin_k
        return img_raw, img_k, img_bin, ch_NIR, ch_Red, ls_bin

class Widget_Kmeans(Widget_Img):
    def __init__(self, img):
        super().__init__(img)
        self.setMouseTracking(True)
        self.pos = None
        self.zoom = 1
        self.imgC = img.shape[2]
        self.img_k = None
        self.img_bin, self.img_temp, self.img_bin_sm = None, None, None
        self.nK, self.center = 0, None
        # img process
        self.img_deshad = np.ones((img.shape[0], img.shape[1]))
        self.val_shad, self.val_shad_tp = 0, 0
        self.val_sm, self.val_sm_tp = 0, 0
        self.ls_bin = [0]
        #
        self.initUI()
    def initUI(self):
        self.show()
    def paintEvent(self, paint_event):
        painter = QPainter(self)
        super().paintImage(painter)
        painter.end()
    def do_kmeans(self, k=3, ch_NIR=1, ch_Red=0):
        self.nK = k
        self.img_k, self.center = get_kmeans(self.img_raw, k, ch_NIR, ch_Red)
    def set_binarize(self, list):
        self.ls_bin = np.array(list)
        self.do_process()
    def set_deshadow(self, value=0):
        self.val_shad_tp = value
        self.do_process()
    def set_smooth(self, value=0):
        self.val_sm_tp = value
        self.do_process()
    def do_process(self):
        # binary
        self.do_binarize()
        # smooth
        self.do_smooth()
        # deshadow
        if self.val_shad!=self.val_shad_tp:
            self.val_shad = self.val_shad_tp
            self.do_deshadow()
        self.switch_imgB()
    def do_binarize(self):
        # prop_g = [self.center[i, 0] for i in range(self.center.shape[0])]
        # prop_g = [(self.center[i, 1]-self.center[i, 0]) for i in range(self.center.shape[0])]
        prop_g = [(self.center[i, 0]-self.center[i, 1])/self.center[i, :].sum() for i in range(self.center.shape[0])]
        rank_g = np.flip(np.argsort(prop_g), 0)
        try:
            idx_select = rank_g[self.ls_bin]
        except:
            idx_select = []
        self.ls_bin_k = idx_select
        self.img_bin = ((np.isin(self.img_k, self.ls_bin_k))*1).astype(np.int)
        self.img_bin_sm = self.img_bin.copy()
        self.img_temp = self.img_bin.copy()
        self.val_sm = 0
    def do_deshadow(self):
        img_mean = self.img_raw[:,:,:3].mean(axis=2)
        self.img_deshad = (img_mean >= self.val_shad)*1
    def do_smooth(self):
        '''
        '''
        from scipy.signal import convolve2d
        k_blur = np.array((
            [1, 4, 1],\
            [4, 9, 4],\
            [1, 4, 1]), dtype='int')/29

        nSm_diff = self.val_sm_tp - self.val_sm
        self.val_sm = self.val_sm_tp
        if nSm_diff>0:
            n_really_do = nSm_diff
        elif nSm_diff<0:
            n_really_do = self.val_sm_tp
            self.img_temp = self.img_bin.copy()
        else:
            n_really_do = 0
        # print("really do : %d, cur sm : %d" % (n_really_do, self.nSm))
        for i in range(n_really_do):
            self.img_temp = convolve2d(self.img_temp, k_blur, mode='same')
        self.img_bin_sm[self.img_temp>0.5] = 1
        self.img_bin_sm[self.img_temp<=0.5] = 0
        self.img_bin_sm = self.img_bin_sm.astype(np.int)
    def switch_imgVis(self):
        super().make_rgb_img(self.img_vis)
        self.repaint()
        self.updateMag()
    def switch_imgK(self):
        super().make_idx8_img(self.img_k, self.nK)
        self.repaint()
        self.updateMag()
    def switch_imgB(self):
        img_b = np.multiply(self.img_bin_sm, self.img_deshad)
        super().make_bin_img(img_b)
        self.repaint()
        self.updateMag()
    def rotate_CW(self):
        self.img_raw = np.rot90(self.img_raw, 3).copy()
        self.img_vis = np.rot90(self.img_vis, 3).copy()
        self.img_k = np.rot90(self.img_k, 3).copy()
        self.img_bin = np.rot90(self.img_bin, 3).copy()
        self.img_temp = np.rot90(self.img_temp, 3).copy()
        self.img_bin_sm = np.rot90(self.img_bin_sm, 3).copy()
        self.img_deshad = np.rot90(self.img_deshad, 3).copy()
        self.switch_imgB()
    def rotate_CCW(self):
        self.img_raw = np.rot90(self.img_raw, 1).copy()
        self.img_vis = np.rot90(self.img_vis, 1).copy()
        self.img_k = np.rot90(self.img_k, 1).copy()
        self.img_bin = np.rot90(self.img_bin, 1).copy()
        self.img_temp = np.rot90(self.img_temp, 1).copy()
        self.img_bin_sm = np.rot90(self.img_bin_sm, 1).copy()
        self.img_deshad = np.rot90(self.img_deshad, 1).copy()
        self.switch_imgB()
    def mouseMoveEvent(self, event):
        self.updateMag()
        self.repaint()
    def mousePressEvent(self, event):
        if event.button() == Qt.RightButton:
            self.zoom = (self.zoom+1)%3
            self.mouseMoveEvent(event)
    def updateMag(self):
        pos = self.mapFromGlobal(QCursor().pos())
        if self.zoom!=0:
            magnifying_glass(self, pos, area=200, zoom=self.zoom*2)
        else:
            self.setCursor(QCursor(Qt.ArrowCursor))
    def getImages(self):
        self.img_bin_sm = np.multiply(self.img_bin_sm, self.img_deshad).astype(np.uint8)
        return self.img_raw, self.img_k, self.img_bin_sm

def get_kmeans(img, k=3, ch_NIR=1, ch_Red=0):
    '''
    '''
    import cv2
    # data type conversion for opencv
    img = img[:,:,[ch_NIR, ch_Red]].copy()
    img_max, img_min = img.max(axis=(0, 1)), img.min(axis=(0, 1))-(1e-8)
    img = (img-img_min)/(img_max-img_min)
    img_z = img.reshape((-1, img.shape[2])).astype(np.float32)
    # define criteria, number of clusters(K) and apply kmeans()
    criteria = (cv2.TERM_CRITERIA_EPS + cv2.TERM_CRITERIA_MAX_ITER, 50, 1.0)
    param_k = dict(data=img_z,\
                   K=k,\
                   bestLabels=None,\
                   criteria=criteria,\
                   attempts=30,\
                   flags=cv2.KMEANS_PP_CENTERS)
                   # KMEANS_RANDOM_CENTERS
    _, img_k_temp, center = cv2.kmeans(**param_k)
    # Convert back
    img_k = img_k_temp.astype(np.uint8).reshape((img.shape[0], -1))
    # return
    return img_k, center



import numpy as np
import copy, os
from .GUI_ClassFun import *
from .CPU_Agents import *
from PyQt5.QtWidgets import *
from PyQt5.QtCore import *
from PyQt5.QtGui import *

class Panel_Output(QWidget):
    def __init__(self, **params):
        '''
        '''
        super().__init__()
        self.setFocusPolicy(Qt.StrongFocus)
        self.setFocus()
        self.update()
        self.field = Field(**params)
        self.layout = QHBoxLayout()
        '''left side'''
        self.wg_img = Widget_Seg(field=self.field)
        '''right side'''
        self.pn_right = QWidget()
        self.lo_right = QVBoxLayout()
        # Config
        self.gr_config = QGroupBox("Config")
        self.lo_config = QVBoxLayout()
        self.gr_grid = QGroupBox("Grid Coef. = 0.2")
        self.lo_grid = QVBoxLayout()
        self.sl_grid = QSlider(Qt.Horizontal)
        self.gr_smc = QGroupBox("Center = 0")
        self.lo_smc = QVBoxLayout()
        self.sl_smc = QSlider(Qt.Horizontal)
        # Display
        self.gr_dis = QGroupBox("Display")
        self.lo_dis = QHBoxLayout()
        self.rb_srgb = QRadioButton("Selected RGB (A)")
        self.rb_rgb = QRadioButton("RGB (S)")
        # Output
        self.gr_out = QGroupBox("Output")
        self.lo_out = QGridLayout()
        self.lb_project = QLabel("Prefix")
        self.fd_project = QLineEdit("GRID")
        self.lb_output = QLabel("Output Path")
        self.fd_output = QLineEdit(os.path.expanduser("~"))
        self.bt_output = QPushButton("Browse")
        '''ui'''
        self.initUI()
    def initUI(self):
        '''config (right)'''
        # components
        self.sl_grid.setMinimum(0)
        self.sl_grid.setMaximum(10)
        self.sl_grid.setValue(2)
        self.sl_grid.setTickInterval(2)
        self.sl_grid.setTickPosition(QSlider.TicksBelow)
        self.sl_grid.valueChanged.connect(self.change_grid)
        self.sl_smc.setMinimum(0)
        self.sl_smc.setMaximum(10)
        self.sl_smc.setValue(0)
        self.sl_smc.setTickInterval(2)
        self.sl_smc.setTickPosition(QSlider.TicksBelow)
        self.sl_smc.valueChanged.connect(self.change_smc)
        # layout
        self.lo_grid.addWidget(self.sl_grid)
        self.gr_grid.setLayout(self.lo_grid)
        self.lo_smc.addWidget(self.sl_smc)
        self.gr_smc.setLayout(self.lo_smc)
        self.lo_config.addWidget(self.gr_grid)
        self.lo_config.addWidget(self.gr_smc)
        self.gr_config.setLayout(self.lo_config)
        '''display (right)'''
        # components
        self.rb_srgb.setChecked(True)
        self.rb_srgb.toggled.connect(self.wg_img.switch_imgSVis)
        self.rb_rgb.toggled.connect(self.wg_img.switch_imgVis)
        # layout
        self.lo_dis.addWidget(self.rb_srgb)
        self.lo_dis.addWidget(self.rb_rgb)
        self.gr_dis.setLayout(self.lo_dis)
        '''output (right)'''
        # components
        font = self.fd_project.font()
        font.setPointSize(25)
        fm = QFontMetrics(font)
        self.fd_project.setFixedHeight(fm.height())
        self.fd_output.setFixedHeight(fm.height())
        self.bt_output.clicked.connect(self.assign_PathOut)
        # layout
        self.lo_out.addWidget(self.lb_project, 0, 0)
        self.lo_out.addWidget(self.fd_project, 0, 1)
        self.lo_out.addWidget(self.lb_output, 1, 0)
        self.lo_out.addWidget(self.fd_output, 1, 1)
        self.lo_out.addWidget(self.bt_output, 1, 2)
        self.gr_out.setLayout(self.lo_out)
        '''layout'''
        # left
        # NONE
        # right
        self.lo_right.addWidget(self.gr_config)
        self.lo_right.addWidget(self.gr_dis)
        self.lo_right.addWidget(self.gr_out)
        self.pn_right.setLayout(self.lo_right)
        # policy
        policy_right = QSizePolicy(QSizePolicy.Preferred, QSizePolicy.Preferred)
        policy_right.setHorizontalStretch(1)
        self.pn_right.setSizePolicy(policy_right)
        policy_left = QSizePolicy(QSizePolicy.Preferred, QSizePolicy.Preferred)
        policy_left.setHorizontalStretch(2)
        self.wg_img.setSizePolicy(policy_left)
        # assemble
        self.layout.addWidget(self.wg_img)
        self.layout.addWidget(self.pn_right)
        self.setLayout(self.layout)
        self.show()
    def change_grid(self):
        '''
        '''
        value = self.sl_grid.value()
        self.gr_grid.setTitle("Grid Coef. = %.2f" %(value/10))
        self.change_config()
    def change_smc(self):
        '''
        '''
        value = self.sl_smc.value()
        self.gr_smc.setTitle("Center = %.2f" %(value/10))
        self.change_config()
    def change_config(self):
        val_grid = (self.sl_grid.value()/10)
        val_center = (self.sl_smc.value()/10)
        self.field.cpu_seg(coef_grid=val_grid, center=val_center)
        self.wg_img.repaint()
    def keyPressEvent(self, event):
        if event.key() == Qt.Key_A:
            self.rb_srgb.setChecked(True)
        elif event.key() == Qt.Key_S:
            self.rb_rgb.setChecked(True)
    def assign_PathOut(self):
        path = QFileDialog().getExistingDirectory(self, "", "", QFileDialog.ShowDirsOnly)
        self.fd_output.setText(path)
    def paint_grid(self, qimg):
        """
        """
        pen = QPen()
        pen.setWidth(1)
        pen.setColor(Qt.red)
        painter = QPainter(qimg)
        painter.setPen(pen)
        painter.setBrush(Qt.transparent)
        for row in range(self.field.nrow):
            for col in range(self.field.ncol):
                agent = self.field.get_agent(row, col)
                rect = agent.get_rect()
                painter.drawRect(rect)
        painter.end()
    def out_raw(self, path):
        """
        """
        img = self.wg_img.img_raw[:,:,:3].copy()
        self.wg_img.make_rgb_img(img)
        qimg = self.wg_img.qimg
        # self.paint_grid(qimg)
        save_img(qimg, path+"_raw")
        self.wg_img.switch_imgSVis()
    def out_rgb(self, path):
        """
        """
        img = self.wg_img.img_raw[:,:,:3].copy()
        self.wg_img.make_rgb_img(img)
        qimg = self.wg_img.qimg
        self.paint_grid(qimg)
        save_img(qimg, path+"_rgb")
        self.wg_img.switch_imgSVis()
    def out_k(self, path):
        """
        """
        img = self.wg_img.img_k.copy()
        self.wg_img.make_idx8_img(img, img.max()+1)
        qimg = self.wg_img.qimg
        self.paint_grid(qimg)
        save_img(qimg, path+"_kmeans")
        self.wg_img.switch_imgSVis()
    def out_idx(self, path):
        img = self.wg_img.img_idx.copy()
        self.wg_img.make_gray_img(img)
        qimg = self.wg_img.qimg
        self.paint_grid(qimg)
        save_img(qimg, path+"_index")
        self.wg_img.switch_imgSVis()
    def out_seg(self, path):
        """
        """
        self.wg_img.make_rgb_img(self.wg_img.img_seg)
        qimg = self.wg_img.qimg
        self.paint_grid(qimg)
        save_img(qimg, path+"_seg")
        self.wg_img.switch_imgSVis()
    def out_bin(self, path):
        """
        """
        self.wg_img.make_bin_img(self.wg_img.img_bin)
        qimg = self.wg_img.qimg
        self.paint_grid(qimg)
        save_img(qimg, path+"_kmean")
        self.wg_img.switch_imgSVis()
    def output(self):
        """
        """
        path_out = self.fd_output.text()+"/"+self.fd_project.text()
        '''figure'''
        self.out_raw(path=path_out)
        self.out_rgb(path=path_out)
        self.out_k(path=path_out)
        self.out_idx(path=path_out)
        self.out_seg(path=path_out)
        self.out_bin(path=path_out)
        '''dataframe'''
        df = self.field.get_DF()
        # NDVI
        idx = self.field.get_index(ch_1=3, ch_2=0, isContrast=True, name_index="NDVI")
        df = pd.merge(df, idx, on='var', how='left')
        # GNDVI
        idx = self.field.get_index(ch_1=3, ch_2=1, isContrast=True, name_index="GNDVI")
        df = pd.merge(df, idx, on='var', how='left')
        # NDGI
        idx = self.field.get_index(ch_1=1, ch_2=0, isContrast=True, name_index="NDGI")
        df = pd.merge(df, idx, on='var', how='left')
        # CNDVI
        idx = self.field.get_index(ch_1=3, ch_2=0, ch_3=1, isThree=True, name_index="CNDVI")
        df = pd.merge(df, idx, on='var', how='left')
        # RVI
        idx = self.field.get_index(ch_1=3, ch_2=0, isRatio=True, name_index="RVI")
        df = pd.merge(df, idx, on='var', how='left')
        # GRVI
        idx = self.field.get_index(ch_1=3, ch_2=1, isRatio=True, name_index="GRVI")
        df = pd.merge(df, idx, on='var', how='left')
        # channels
        for i in range(self.field.n_ch):
            idx = self.field.get_index(ch_1=i, isSingle=True, name_index="ch_%d"%i)
            df = pd.merge(df, idx, on='var', how='left')
        # clusters
        idx = self.field.get_index(ch_1=self.field.ch_nir, ch_2=self.field.ch_red, isContrast=True, name_index="CLUSTER_INDEX")
        df = pd.merge(df, idx, on='var', how='left')
        idx = self.field.get_cluster()
        df = pd.merge(df, idx, on='var', how='left')
        # export
        df.to_csv(path_out+"_data.csv", index=False)

class Widget_Seg(Widget_Img):
    def __init__(self, field):
        '''
        '''
        super().__init__(field.img_raw)
        self.setMouseTracking(True)
        self.zoom = 1
        '''attr'''
        # basic
        self.field = field
        self.agents_reset = copy.deepcopy(self.field.agents)
        self.img_raw = field.img_raw
        self.img_bin = field.img_bin
        self.img_k = field.img_k
        # painter
        self.is_fit_width = False
        self.pt_st_img = 0
        self.ratio = 0
        # mouse
        self.agent_click = None
        self.dir = None
        # calculate index image
        ch_1 = self.field.ch_nir
        ch_2 = self.field.ch_red
        self.img_idx = (self.img_raw[:,:,ch_1]-self.img_raw[:,:,ch_2])/(self.img_raw[:,:,ch_1]+self.img_raw[:,:,ch_2]+1e-8).copy()
        self.img_idx = ((self.img_idx)*255).astype(np.uint8)
        # ui
        self.initUI()
    def initUI(self):
        '''generate seg image'''
        img_temp = self.img_bin.reshape(self.img_bin.shape[0], self.img_bin.shape[1], 1)
        self.img_seg = np.multiply(self.img_raw[:,:,:3], img_temp).copy()
        # self.img_seg[(self.img_seg.mean(axis=2)==0), :] = self.img_seg.max()
        self.img_seg[(self.img_seg.mean(axis=2)==0), :] = 255
        self.switch_imgSVis()
        self.show()
    def mousePressEvent(self, event):
        pos = event.pos()
        for row in range(self.field.nrow):
            for col in range(self.field.ncol):
                agent = self.field.get_agent(row, col)
                rect = agent.get_rect()
                if self.is_fit_width:
                    self.ratio = self.width()/self.qimg.width()
                    rec_agent = QRect(rect.x()*self.ratio, rect.y()*self.ratio+self.pt_st_img, rect.width()*self.ratio, rect.height()*self.ratio)
                else:
                    self.ratio = self.height()/self.qimg.height()
                    rec_agent = QRect(rect.x()*self.ratio+self.pt_st_img, rect.y()*self.ratio, rect.width()*self.ratio, rect.height()*self.ratio)
                if rec_agent.contains(pos):
                    bd_W = rec_agent.x()
                    bd_N = rec_agent.y()
                    bd_E = bd_W + rec_agent.width()
                    bd_S = bd_N + rec_agent.height()
                    dis_W = abs(pos.x()-bd_W)
                    dis_N = abs(pos.y()-bd_N)
                    dis_E = abs(pos.x()-bd_E)
                    dis_S = abs(pos.y()-bd_S)
                    # print("W:%.2f, N:%.2f, E:%.2f, S:%.2f" %(dis_W, dis_N, dis_E, dis_S))
                    dir_idx = np.argmin(np.array([dis_N, dis_W, dis_S, dis_E]))
                    if dir_idx==0:
                        self.dir = Dir.NORTH
                    elif dir_idx==1:
                        self.dir = Dir.WEST
                    elif dir_idx==2:
                        self.dir = Dir.SOUTH
                    elif dir_idx==3:
                        self.dir = Dir.EAST
                    self.agent_click = agent
                    break
        # mag module
        if event.button() == Qt.RightButton:
            self.zoom = (self.zoom+1)%3
            self.mouseMoveEvent(event)
    def mouseMoveEvent(self, event):
        pos = event.pos()
        if event.buttons() == Qt.LeftButton:
            # adjust the border
            if self.is_fit_width:
                if self.dir==Dir.NORTH or self.dir==Dir.SOUTH:
                    value = (pos.y()-self.pt_st_img)/self.ratio
                elif self.dir==Dir.WEST or self.dir==Dir.EAST:
                    value = pos.x()/self.ratio
            else:
                if self.dir==Dir.NORTH or self.dir==Dir.SOUTH:
                    value = pos.y()/self.ratio
                elif self.dir==Dir.WEST or self.dir==Dir.EAST:
                    value = (pos.x()-self.pt_st_img)/self.ratio
            self.agent_click.set_border(self.dir, value)
        # mag module
        if self.zoom!=0:
            magnifying_glass(self, pos, area=200, zoom=self.zoom*2)
        else:
            self.setCursor(QCursor(Qt.ArrowCursor))
        self.repaint()
    def paintEvent(self, paint_event):
        painter = QPainter(self)
        super().paintImage(painter)
        pen = QPen()
        pen.setWidth(1)
        pen.setColor(Qt.red)
        painter.setPen(pen)
        painter.setBrush(Qt.transparent)
        for row in range(self.field.nrow):
            for col in range(self.field.ncol):
                agent = self.field.get_agent(row, col)
                rect = agent.get_rect()
                pt_x, pt_y = agent.get_coordinate()
                if self.is_fit_width:
                    self.ratio = self.width()/self.qimg.width()
                    rec_agent = QRect(rect.x()*self.ratio, rect.y()*self.ratio+self.pt_st_img, rect.width()*self.ratio, rect.height()*self.ratio)
                    draw_cross(pt_x*self.ratio, pt_y*self.ratio+self.pt_st_img, painter)
                else:
                    self.ratio = self.height()/self.qimg.height()
                    rec_agent = QRect(rect.x()*self.ratio+self.pt_st_img, rect.y()*self.ratio, rect.width()*self.ratio, rect.height()*self.ratio)
                    draw_cross(pt_x*self.ratio+self.pt_st_img, pt_y*self.ratio, painter)
                painter.drawRect(rec_agent)
        painter.end()
    def switch_imgVis(self):
        super().make_rgb_img(self.img_vis)
        self.repaint()
    def switch_imgSVis(self):
        super().make_rgb_img(self.img_seg)
        self.repaint()
    def reset(self):
        self.field.agents = None
        self.field.agents = copy.deepcopy(self.agents_reset)
        self.repaint()



import setuptools
setuptools.setup(name='photo_grid',
      version='0.0.08',
      description='A GUI for field segmentation',
      url='https://github.com/Poissonfish/photo_grid',
      author='James Chen',
      author_email='chun-peng.chen@wsu.edu',
      license='MIT',
      packages=setuptools.find_packages(),
      install_requires=['numpy',
                        'pandas>=0.19.2',
                        'sklearn',
                        'scipy',
                        'opencv-python',
                        'image',
                        'rasterio',
                        'PyQt5',
                        'tqdm'])
